@@ -1,5 +1,7 @@
 #!/usr/bin/env python
-# coding: utf-8
+# -*- coding: utf-8 -*-
+# Copyright (c) 2011-2013 Raphaël Barrois
+# This code is distributed under the two-clause BSD License.
 
 import warnings
 
@@ -498,7 +500,7 @@ class TransitionRunningTestCase(unittest.TestCase):
             state = MyWorkflow()
 
             def __repr__(self):
-                r = u("blé∫")
+                return u("blé∫")
 
         self.MyWorkflow = MyWorkflow
         self.MyWorkflowObject = MyWorkflowObject
@@ -886,9 +888,13 @@ class TransitionHookTestCase(unittest.TestCase):
 
         Matching is performed on (priority, callable, kind) pairs.
         """
-        self.assertItemsEqual(
-            [(priority, fun, kind) for (priority, fun) in expected],
-            [(hook.priority, hook.function, hook.kind) for hook in actual])
+        expected_items = sorted(
+            ((priority, fun, kind) for (priority, fun) in expected),
+            key=lambda t: (t[0], hash(t[1]), t[2]))
+        actual_items = sorted(
+            ((hook.priority, hook.function, hook.kind) for hook in actual),
+            key=lambda t: (t[0], hash(t[1]), t[2]))
+        self.assertEqual(list(expected_items), list(actual_items))
 
     def test_declarations(self):
         obj = self.MyWorkflowObject()
@@ -1209,9 +1215,13 @@ class StateHookTestCase(unittest.TestCase):
 
         Matching is performed on (priority, callable, kind) pairs.
         """
-        self.assertItemsEqual(
-            [(priority, fun, kind) for (priority, fun) in expected],
-            [(hook.priority, hook.function, hook.kind) for hook in actual])
+        expected_items = sorted(
+            ((priority, fun, kind) for (priority, fun) in expected),
+            key=lambda t: (t[0], hash(t[1]), t[2]))
+        actual_items = sorted(
+            ((hook.priority, hook.function, hook.kind) for hook in actual),
+            key=lambda t: (t[0], hash(t[1]), t[2]))
+        self.assertEqual(list(expected_items), list(actual_items))
 
     def test_declarations(self):
         obj = self.MyWorkflowObject()
@@ -1313,6 +1323,42 @@ class HookInheritanceTestCase(unittest.TestCase):
         # after: 2
         self.assertEqual([1, 2], obj.hooks)
 
+    def test_multiple_inherited(self):
+        class WorkflowObject(base.WorkflowEnabled):
+            state = self.MyWorkflow()
+
+            def __init__(self):
+                self.hooks = []
+
+            def seen_hook(self, hook_id):
+                self.hooks.append(hook_id)
+
+            @base.on_leave_state('foo')
+            def hook1(self, *args, **kwargs):
+                self.seen_hook(1)
+
+        class MyFirstSub(WorkflowObject):
+            pass
+
+        class MySecondSub(WorkflowObject):
+            pass
+
+        o1 = MyFirstSub()
+        self.assertEqual(self.MyWorkflow.states.foo, o1.state)
+        o1.foobar()
+        # check: -
+        # before: 1
+        # after: 2
+        self.assertEqual([1], o1.hooks)
+
+        o2 = MySecondSub()
+        self.assertEqual(self.MyWorkflow.states.foo, o2.state)
+        o2.foobar()
+        # check: -
+        # before: 1
+        # after: 2
+        self.assertEqual([1], o2.hooks)
+
     def test_extra_hook(self):
         class MySubWorkflowObject(self.MyWorkflowObject):
             @base.before_transition('foobar')
@@ -1343,6 +1389,46 @@ class HookInheritanceTestCase(unittest.TestCase):
         # after: 2
         self.assertEqual([3, 2], obj.hooks)
 
+    def test_multiple_inherited_with_implem(self):
+        class WorkflowObject(base.WorkflowEnabled):
+            state = self.MyWorkflow()
+
+            def __init__(self):
+                self.hooks = []
+
+            def seen_hook(self, hook_id):
+                self.hooks.append(hook_id)
+
+            @base.transition()
+            def foobar(self, *args, **kwargs):
+                pass
+
+            @base.on_leave_state('foo')
+            def hook1(self, *args, **kwargs):
+                self.seen_hook(1)
+
+        class MyFirstSub(WorkflowObject):
+            pass
+
+        class MySecondSub(WorkflowObject):
+            pass
+
+        o1 = MyFirstSub()
+        self.assertEqual(self.MyWorkflow.states.foo, o1.state)
+        o1.foobar()
+        # check: -
+        # before: 1
+        # after: 2
+        self.assertEqual([1], o1.hooks)
+
+        o2 = MySecondSub()
+        self.assertEqual(self.MyWorkflow.states.foo, o2.state)
+        o2.foobar()
+        # check: -
+        # before: 1
+        # after: 2
+        self.assertEqual([1], o2.hooks)
+
 
 
 class ExtendedTransitionImplementationTestCase(unittest.TestCase):
